<?php
/*
 * Plugin Name: Code Studio
 * Plugin URI: https://wpstudio.com/plugins/code-studio
 * Description: Add and control the inclusion of CSS, Javascript, HTML, and more to your WordPress site.
 * Version: 1.1.5
 * Author: Benjamin Bradley
 * Author URI: https://b.enjam.in
 * License: GPLv2 or later
*/

if ( ! defined( 'ABSPATH' ) ) { exit; }

if ( ! class_exists( 'WPSCodeStudio' ) ) :

  final class WPSCodeStudio {
    public $search_tree = false;
    public $allowed_codes = true;
    protected static $_instance = null;

    public static function instance() {
      if ( is_null( self::$_instance )){
        self::$_instance = new self();
      }
      return self::$_instance;
    }

    public function __clone(){}
    public function __wakeup(){}

    public function __construct() {
      add_action( 'init', array( $this, 'register_post_type' ) );
      $this->set_constants();
      include_once( 'inc/functions.php' );
      if ( is_admin() ) {
        add_action('admin_init', array( $this, 'remove_menu_link') );
        include_once( 'inc/admin-screens.php' );
        include_once( 'inc/admin-preview.php' );
        include_once( 'inc/admin-url-rules.php' );
        include_once( 'inc/admin-revisions.php' );
        include_once( 'inc/admin-import.php' );
        include_once( 'inc/admin-shortcodes.php' );
        //include_once( 'inc/admin-choose-theme.php' );
        include_once( 'inc/admin-config.php' );
        include_once( 'inc/admin-warnings.php' );
        /* Include updater. */
        require_once( trailingslashit( plugin_dir_path( __FILE__ ) ) . 'inc/wpstudio_updater.php' );
      }
      add_action( 'wp', array( $this, 'initiate_show_codes' ) );
    }

    function initiate_show_codes() {
      include_once( 'inc/class-show-codes.php' );
      $show_codes = new WPSCodeStudio_ShowCodes();
      $show_codes->set_value('first_page', get_option('home') );
      $show_codes->set_value('upload_dir', WPSCS_UPLOAD_DIR );
      $show_codes->set_value('upload_url', WPSCS_UPLOAD_URL );

      // Get the allowed codes for this specific page
      $uri = $_SERVER['REQUEST_URI'];
      $wpscs_urls = get_option( 'code-studio-urls' );
      $allowed_codes = $show_codes->url_rules( $uri, $wpscs_urls );
      $allowed_codes = $show_codes->wp_conditional_tags( $allowed_codes, $wpscs_urls );

      // Print shortcodes
      $search_tree = get_option( 'code-studio-tree' );
      $show_codes->set_value('search_tree_raw', $search_tree );
      $show_codes->add_shortcodes();

      // Get the filtered search tree
      $search_tree = $show_codes->filter_search_tree( $search_tree, $allowed_codes );
      if ( isset($_GET['wpscs-preview-id'] ) ) {
        $preview_id = $_GET['wpscs-preview-id'];
        $search_tree = $show_codes->search_tree_for_preview( $search_tree, $preview_id );
      }

      // Print the codes
      $show_codes->set_value('search_tree', $search_tree );
      $show_codes->print_code_actions( $search_tree );
    }

    function remove_menu_link(){
      if ( ! current_user_can('activate_plugins')){
        remove_menu_page('edit.php?post_type=code-studio');
      }
    }

    function set_constants(){
      $dir = wp_upload_dir();
      define( 'WPSCS_PREVIEW_PREFIX', 'wpscs_preview-');
      define('WPSCS_VERSION', '1.1.5');
      define('WPSCS_UPLOAD_DIR', $dir['basedir'] . '/code-studio');
      define('WPSCS_UPLOAD_URL', $dir['baseurl'] . '/code-studio');
      define('WPSCS_PLUGIN_FILE', __FILE__);
    }

    public function register_post_type() {
      $labels = array(
          'name' => _x( 'Code Studio', 'post type general name'),
          'singular_name' => _x( 'Code Studio', 'post type singular name'),
          'menu_name' => _x( 'Code Studio', 'admin menu'),
          'name_admin_bar' => _x( 'Code Studio', 'add new on admin bar'),
          'add_new' => _x( 'Add Custom Code', 'add new'),
          'add_new_item' => __( 'Add Custom Code'),
          'new_item' => __( 'New Custom Code'),
          'edit_item' => __( 'Edit Custom Code'),
          'view_item' => __( 'View Custom Code'),
          'all_items' => __( 'Code Studio Files'),
          'search_items' => __( 'Search Code Studio Items'),
          'parent_item_colon' => __( 'Parent Custom Code:'),
          'not_found' => __( 'No Custom Code found.'),
          'not_found_in_trash' => __( 'No Custom Code found in Trash.')
      );
      $args = array(
          'labels' => $labels,
          'description' => __( 'Code Studio Code' ),
          'public' => false,
          'publicly_queryable' => false,
          'show_ui' => true,
          'show_in_menu' => true,
          'menu_position' => 100,
          'menu_icon' => 'dashicons-editor-code',
          'query_var' => false,
          'rewrite' => array( 'slug' => 'code-studio' ),
          'capability_type' => 'post',
          'has_archive' => true,
          'hierarchical' => false,
          'menu_position' => null,
          'supports' => array( 'title', 'revisions' )
      );
      register_post_type( 'code-studio', $args );
    }
  }
endif;

function WPSCodeStudio() {
  return WPSCodeStudio::instance();
}

WPSCodeStudio();

register_activation_hook( __FILE__, 'wps_code_studio_activation' );
function wps_code_studio_activation() {
    $wpscs_urls = get_option( 'code-studio-urls' );
    if ( $wpscs_urls !== false ) {
        return false;
    }
    // get all codes
    $posts = query_posts( 'post_type=code-studio&post_status=publish&nopaging=true' );
    // return if no codes available
    if ( ! is_array( $posts ) || count( $posts ) == 0 ) {
        return false;
    }
    $ids = array();
    foreach ( $posts as $_post ) {
        if ( get_post_meta( $_post->ID, '_active', true ) === 'no' ) {
            // return if the code is not active
            continue;
        }
        $options = wpscs_get_options( $_post->ID );
        if ( ! isset( $options['language'] ) ) continue;
        if ( $options['language'] ==  'html' ) {
            $file = $_post->ID;
        } else {
            $file = $_post->ID . '.' . $options['language'];
        }
        $ids[] = $file;
    }
    update_option( 'code-studio-urls', array('all' => $ids ) );
}

function closed_meta_boxes( $closed ){
      if ( false === $closed ) {
        $closed = array('url-rules', 'revisionsdiv');
      }
      return $closed;
    }
add_filter('get_user_option_closedpostboxes_code-studio', 'closed_meta_boxes');