<?php
if ( ! defined( 'ABSPATH' ) ) { exit; }

class WPSCodeStudio_ChooseTheme {
    var $default = '';

    public function __construct() {
        add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
        add_action( 'wpscs_settings_form', array( $this, 'wpscs_settings_form' ), 11 );
        add_filter( 'wpscs_settings_default', array( $this, 'wpscs_settings_default' ) );
        add_filter( 'wpscs_settings_save', array( $this, 'wpscs_settings_save' ) );
    }

    function wpscs_settings_default( $defaults ) {
        return array_merge( $defaults, array( 'wpscs_editor_theme' => $this->default ) );
    }

    function wpscs_settings_save( $data ) {
        $editor = isset($_POST['wpscs_editor_theme'] ) ? trim($_POST['wpscs_editor_theme']) : $this->default;
        if ( ! in_array( $editor, $this->get_themes() ) ) {
            $editor = $this->default;
        }
        return array_merge( $data, array('wpscs_editor_theme' => $editor ) );
    }

    function wpscs_settings_form() {

        $settings = get_option('wpscs_settings');
        $the_theme = isset($settings['wpscs_editor_theme']) ? $settings['wpscs_editor_theme'] : $this->default;

        $themes = $this->get_themes();

        $help = '<span class="dashicons dashicons-editor-help" rel="tipsy" title="Changing the editor\'s theme will affect all the users, not only you."></span>';
        ?>
        <tr>
        <th scope="row"><label for="wpscs_editor_theme">Editor theme <?php echo $help; ?></label></th>
        <td><select onchange="selectTheme()" name="wpscs_editor_theme" id="wpscs_editor_theme"><?php 
        foreach( $themes as $_theme ) {
            $selected = ( $_theme == $the_theme ) ? ' selected' : '';
            echo "\t\t\t" . '<option value="'.$_theme.'"'.$selected.'>'.$_theme.'</option>' . PHP_EOL;
        }
        ?>
        </select> <br />
        <?php $this->theme_demo(); ?>
        </td>
        </tr>
        <?php
    }

    public function admin_enqueue_scripts( $hook ) {
        $screen = get_current_screen();

        if ( $screen->post_type != 'code-studio' ) 
            return false;

        if ( $hook != 'code-studio_page_code-studio-config' ) 
            return false;

        $a = plugins_url( '/', WPSCS_PLUGIN_FILE). 'assets';
        $cm = $a . '/codemirror';
        $v = WPSCS_VERSION; 

        wp_enqueue_script( 'addon_merge_match_patch', $cm . '/diff_match_patch.js', array( 'jquery' ), $v, false );
        wp_enqueue_script( 'codemirror', $cm . '/codemirror-compressed.js', array( 'jquery' ), $v, false);
        wp_enqueue_style( 'codemirror', $cm . '/codemirror-compressed.css', array(), $v );

        $themes = $this->get_themes();

        foreach( $themes as $_theme ) {
            if ( $_theme == 'default' ) continue;
            wp_enqueue_style( 'cmt-' . $_theme, $cm . '/theme/' . $_theme . '.css', array(), $v );
        }
    }

    function get_themes() {
       return array(
            'default',
            '3024-day',
            '3024-night',
            'abcdef',
            'ambiance',
            'base16-dark',
            'base16-light',
            'bespin',
            'blackboard',
            'cobalt',
            'colorforth',
            'dracula',
            'eclipse',
            'elegant',
            'erlang-dark',
            'hopscotch',
            'icecoder',
            'isotope',
            'lesser-dark',
            'liquibyte',
            'material',
            'mbo',
            'mdn-like',
            'midnight',
            'monokai',
            'neat',
            'neo',
            'night',
            'paraiso-dark',
            'paraiso-light',
            'pastel-on-dark',
            'railscasts',
            'rubyblue',
            'seti',
            'solarized',
            'the-matrix',
            'tomorrow-night-bright',
            'tomorrow-night-eighties',
            'ttcn',
            'twilight',
            'vibrant-ink',
            'xq-dark',
            'xq-light',
            'yeti',
            'zenburn',
        );

    }

    function theme_demo() {
    ?> 
<div style="margin: 5px 0;">Theme preview:</div>
<textarea id="theme_demo" name="theme_demo">
.example {
    color: #eee;
}
h1.example-2 {
    margin-top: 10px !important;
    font-size: 1.3em;
    margin: 1em 0;
    display: block;
    font-weight: 600;
}
</textarea>

        <script>
          var editor = CodeMirror.fromTextArea(document.getElementById("theme_demo"), {
            lineNumbers: true,
            mode: 'css',
          });
          var input = document.getElementById("wpscs_editor_theme");
          function selectTheme() {
            var theme = input.options[input.selectedIndex].textContent;
            editor.setOption("theme", theme);
          }
          window.onload = selectTheme();
        </script>

    <?php
    }

}

return new WPSCodeStudio_ChooseTheme();
