<?php
if ( ! defined( 'ABSPATH' ) ) { exit; }

class WPSCodeStudio_AdminConfig {

    var $settings_default;

    var $settings;

    public function __construct() {
        $settings_default = apply_filters( 'wpscs_settings_default', array() );
        $settings = get_option('wpscs_settings');
        if ( $settings == false ) {
            $settings = $settings_default;
        } else {
            foreach( $settings_default as $_key => $_value ) {
                if ( ! isset($settings[$_key] ) ) {
                    $settings[$_key] = $_value;
                }
            }
        }
        $this->settings = $settings;
        $this->settings_default = $settings_default;

        add_action( 'admin_menu', array( $this, 'admin_menu' ) );
        add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
    }

    function admin_menu() {
        $menu_slug = 'edit.php?post_type=code-studio';
        add_submenu_page( $menu_slug, __('Settings'), __('Settings'), 'manage_options', 'code-studio-config', array( $this, 'config_page' ) );
    }

    public function admin_enqueue_scripts( $hook ) {
        $screen = get_current_screen();
        if ( $screen->post_type != 'code-studio' )
            return false;

        if ( $hook != 'code-studio_page_code-studio-config' )
            return false;

        $a = plugins_url( '/', WPSCS_PLUGIN_FILE). 'assets';
        $v = WPSCS_VERSION;

        wp_enqueue_script( 'tipsy', $a . '/jquery.tipsy.js', array('jquery'), $v, false );
        wp_enqueue_style( 'tipsy', $a . '/tipsy.css', array(), $v );
    }

    function config_page() {
        if ( isset( $_GET['tab'] ) && $_GET['tab'] == 'debug' ) {
            $this->debug_tab();
            return;
        }
        if ( isset( $_GET['tab'] ) && $_GET['tab'] == 'resources' ){
            $this->resources_tab();
            return;
        }
        if ( isset( $_POST['wpscs_settings-nonce'] ) ) {
            check_admin_referer('wpscs_settings', 'wpscs_settings-nonce');
            $data = apply_filters( 'wpscs_settings_save', array() );
            update_option( 'wpscs_settings', $data );
        } else {
            $data = $this->settings;
        }

        ?>
        <div class="wrap">
        <?php $this->config_page_header('general'); ?>
        <form action="<?php echo admin_url('edit.php'); ?>?post_type=code-studio&page=code-studio-config" id="wpscs_settings" method="post">
        <table class="form-table">
                <?php do_action( 'wpscs_settings_form' ); ?>
            <tr>
            <th>&nbsp;</th>
            <td>
            <input type="submit" name="Submit" class="button-primary" value="<?php _e('Save'); ?>" />
            <?php wp_nonce_field('wpscs_settings', 'wpscs_settings-nonce', false); ?>
            </td>
            </tr>
        </table>
        </form>
        </div>
        <?php
    }

    function config_page_header( $tab = 'general' ) {
        $url = '?post_type=code-studio&page=code-studio-config';
        $active = array('general' => '', 'resources' => '', 'debug' => '');
        $active[$tab] = 'nav-tab-active';
        ?>
        <h1 class="top-banner"><?php _e('Code Studio by WPStudio'); ?></h1>
        <h2 class="nav-tab-wrapper woo-nav-tab-wrapper">
            <a href="<?php echo $url; ?>" class="nav-tab <?php echo $active['general']; ?>"><?php echo __('General Settings', 'wpscs'); ?></a>
            <a href="<?php echo $url; ?>&tab=resources" class="nav-tab <?php echo $active['resources']; ?>"><?php echo __('Examples & Help', 'wpscs'); ?></a>
            <?php /*
            <a href="<?php echo $url; ?>&tab=debug" class="nav-tab <?php echo $active['debug']; ?>"><?php echo __('Debug', 'wpscs'); ?></a>
            */ ?>
        </h2>
        <?php
    }

    function resources_tab(){
        ?>
        <div class="wrap">

        <?php $this->config_page_header('resources'); ?>

        <div id="resources" style="display: block;">
            <h2>Resources and Helpful Tips</h2>
            <hr />
            <p>The following example codes are what you can put into the Code areas of each individual Code Studio entry.</p>
            <h4>Example CSS Code</h4>
            <pre>
            .css-class {
                color: #1dfd3e;
                font-size: 18px;
            }
            </pre>
            <h4>Example Javascript Code</h4>
            <pre>
            jQuery(document).ready(function( $ ){
                // Put your script code here
            });
            </pre>
            <h4>Example HTML Code</h4>
            <pre>
            &#x3c;script src="https://ajax.googleapis.com/ajax/libs/jquery/3.1.1/jquery.min.js"&#x3e;&#x3c;/script&#x3e;
            </pre>
            <h4>How to use the "shortcode" option"</h4>
            <ul>
            <li>You can use it in a post/page as [wpscs id="shortcode_id"].</li>
            <li>You can also add variables (a single variable) by using a shortcode that looks like [wpscs id="shortcode_id" variable="Bob"]</li>
            <li>Then in the content area of the HTML Code Studio entry you can type:</li>
            <li><pre>There is an amazing coder named {$variable} who lives in the sea.</pre></li>
            </ul>
        </div>
        </div>
        <?php
    }

    function debug_tab() {
        $wpscs_for_url = '';
        if ( isset( $_POST['wpscs_for_url'] ) ) {
            $wpscs_for_url = $_POST['wpscs_for_url'];
        }
        include_once( 'class-show-codes.php' );
        $show_codes = new WPSCodeStudio_ShowCodes();
        $show_codes->set_value('first_page', get_option('home') );
        $show_codes->set_value('upload_dir', WPSCS_UPLOAD_DIR );
        $show_codes->set_value('upload_url', WPSCS_UPLOAD_URL );

        $uri = $wpscs_for_url;

        $wpscs_urls = get_option( 'code-studio-urls' );
        $allowed_codes = $show_codes->url_rules( $uri, $wpscs_urls );

        $search_tree = get_option( 'code-studio-tree' );
        $filtered_search_tree = $show_codes->filter_search_tree( $search_tree, $allowed_codes );

        // WordPress environment
        $report = '';
        $report .= 'Home URL: ' . get_option( 'home' ) . PHP_EOL;
        $report .= 'Site URL: ' . get_option( 'siteurl' ) . PHP_EOL;
        $report .= 'WP Version: ' . get_bloginfo('version') . PHP_EOL;
        $report .= 'WPSCS Version: ' . WPSCS_VERSION . PHP_EOL;
        $report .= 'WP Multisite: ' . var_export(is_multisite(), true). PHP_EOL;
        $report .= 'WP Memory Limit: ' . @ini_get( 'memory_limit' ). PHP_EOL;

        ob_start();
        echo '### WordPress Environment ###'. PHP_EOL;
        echo $report . PHP_EOL;
        if ( ! empty( $wpscs_for_url ) ) {
            echo '### Report generated for: ' . PHP_EOL;
            echo '   ' . $wpscs_for_url . PHP_EOL;
        }
        echo '### All codes ###' . PHP_EOL;
        print_r( $wpcs_urls );
        if ( ! empty( $wpscs_for_url ) ) {
            echo '### Allowed codes ###' . PHP_EOL;
            print_r( $allowed_codes );
        }
        echo '### Search tree ###' . PHP_EOL;
        print_r( $search_tree );
        if ( ! empty( $wpscs_for_url ) ) {
            echo '### Filtered Search tree ###' . PHP_EOL;
            print_r( $filtered_search_tree );
        }
        $report = ob_get_contents();
        ob_end_clean();

        ?>

<script type="text/javascript">
    jQuery( document ).ready( function( $ ) {

        $("#debug-report textarea").focus().select();

    });
        </script>

        <div class="wrap">

        <?php $this->config_page_header('debug'); ?>

        <div id="debug-report" style="display: block;">
            <div style="margin-bottom: 10px;">Please copy and paste this information when contacting support:</div>
            <textarea readonly="readonly"><?php echo $report; ?></textarea>
            <form method="post">
            <table class="form-table">
                <tr>
                <td>
                <input type="text" name="wpscs_for_url" value="<?php echo $wpscs_for_url; ?>" style="width: 100%;" placeholder="URL for which you want to generate the report" />
                </td><td>
                <input type="submit" name="Submit" class="button-primary" value="<?php _e('Regenerate the debug info'); ?>" />
                </td>
                </tr>
            </table>
            </form>
        </div>
        </div>
        <?php
    }
}

return new WPSCodeStudio_AdminConfig();
