<?php
if ( ! defined( 'ABSPATH' ) ) { exit; }

class WPSCodeStudio_Preview {
    var $default = 3600;

    public function __construct() {
        $actions = array(
            'add_meta_boxes' => 'add_meta_boxes',
            'wp_ajax_wpscs_preview_save' => 'wp_ajax_wpscs_preview_save',
            'admin_enqueue_scripts' => 'admin_enqueue_scripts',
        );
        foreach( $actions as $_key => $_value ) {
            add_action( $_key, array( $this, $_value ) );
        }

        add_action( 'wpscs_settings_form', array( $this, 'wpscs_settings_form' ), 11 );
        add_filter( 'wpscs_settings_default', array( $this, 'wpscs_settings_default' ) );
        add_filter( 'wpscs_settings_save', array( $this, 'wpscs_settings_save' ) );
    }

    function add_meta_boxes() {
        add_meta_box( 'previewdiv', __('Preview'), array( $this, 'previews_meta_box_callback' ), 'code-studio', 'normal' );
    }

    function previews_meta_box_callback( $post ) {
        $preview_url = get_post_meta( $post->ID, 'preview_url' );
        $preview_url = isset($preview_url[0]) ? $preview_url[0] : '';
        $preview_id = get_post_meta( $post->ID, 'preview_id' );
        $preview_id = isset($preview_id[0]) ? $preview_id[0] : $this->random_str(8) . $post->ID;
        wp_nonce_field( 'previews_meta_box_callback', 'wpscs-preview-nonce' );
?>
<div id="preview-action">
    <div>
    <input type="text" name="preview_url" id="wpscs-preview_url" value="<?php echo $preview_url; ?>" placeholder="Full URL on which to preview the changes ..." />
    <a class="preview button button-primary button-large" id="wpscs-preview">Preview Changes</a>
    </div>
    <input type="hidden" name="wpscs_preview-id" id="wpscs_preview-id" value="<?php echo $preview_id; ?>" />
</div>
<?php

    }

    private function random_str($length, $keyspace = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ') {
        $str = '';
        $max = mb_strlen($keyspace, '8bit') - 1;
        for ($i = 0; $i < $length; ++$i) {
            $str .= $keyspace[rand(0, $max)];
        }
        return $str;
    }

    function wp_ajax_wpscs_preview_save() {

        if ( ! wp_verify_nonce( $_POST['wpscs-preview-nonce'], 'previews_meta_box_callback' ) ) {
            echo 'Error: _nonce is not correct. Refresh your browser and try again';
            die();
        }

        $post_id = $_POST['post_ID'];
        $transient_id = WPSCS_PREVIEW_PREFIX . $_POST['preview_id'];

        foreach( array('post_ID', 'linking', 'type', 'side', 'language', 'preprocessor', 'minify', 'priority') as $_type ) {
            $transient_data[$_type] = $_POST[$_type];
        }
        $transient_data['minify'] = ($transient_data['minify'] == 'true' ) ? '1' : '0';

        $settings = get_option('wpscs_settings');
        $preview_duration = isset($settings['wpscs_duration_preview'] ) ? $settings['wpscs_duration_preview'] : 3600;

        set_transient( $transient_id, $transient_data, $preview_duration );

        $filename = $post_id . '-preview.' . $_POST['language'];
        $result = wpscs_save_code_file( $_POST['content'], $transient_data, $filename );

        update_post_meta( $post_id, 'preview_url', $_POST['preview_url']);
        update_post_meta( $post_id, 'preview_id', $_POST['preview_id'] );

        echo 'success';
        die();
    }

    public function admin_enqueue_scripts( $hook ) {
        $screen = get_current_screen();
        if ( $screen->post_type != 'code-studio' )
            return false;
        if ( $hook != 'post-new.php' && $hook != 'post.php' ) {
            return;
        }
        $a = plugins_url( '/', WPSCS_PLUGIN_FILE). 'assets';
        $v = WPSCS_VERSION;
        wp_enqueue_script( 'wpscs_admin-preview', $a . '/wpscs_preview.js', array('jquery', 'wpscs_admin'), $v, false );
    }

    function wpscs_settings_default( $defaults ) {
        $defaults['wpscs_duration_preview'] = $this->default;
        return $defaults;
    }

    function wpscs_settings_form() {
        $settings = get_option('wpscs_settings');
        $value = isset($settings['wpscs_duration_preview']) ? $settings['wpscs_duration_preview'] : $this->default;
        $help = '<span class="dashicons dashicons-editor-help" rel="tipsy" title="How long will last a preview generated for a code."></span>';
        ?>
        <tr>
        <th scope="row"><label for="wpscs_duration_preview">Duration of Preview <?php echo $help; ?></label></th>
        <td><input name="wpscs_duration_preview" type="text" id="wpscs_duration_preview" value="<?php echo $value; ?>" class="regular-text" /> seconds</td>
        </tr>
        <?php
    }

    function wpscs_settings_save( $data ) {
        $duration = isset($_POST['wpscs_duration_preview'] ) ? $_POST['wpscs_duration_preview'] : $this->default;
        $duration = (int)$duration;
        if ( $duration <= 0 ) {
            $duration = $this->default;
        }

        return array_merge( $data, array('wpscs_duration_preview' => $duration ) );
    }



}

return new WPSCodeStudio_Preview();