<?php
if ( ! defined( 'ABSPATH' ) ) { exit; }

class WPSCodeStudio_Admin {
    private $options = array();
    private $theme = '';
    private $themeselect = 'solarized';
    public function __construct() {
        if ( ! function_exists( 'is_plugin_active' ) ) {
            require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        }
        $this->add_functions();
    }

    function add_functions() {
        $filters = array(
            'manage_code-studio_posts_columns' => 'manage_custom_posts_columns',
        );
        foreach( $filters as $_key => $_value ) {
            add_filter( $_key, array( $this, $_value ) );
        }

        $actions = array(
            'admin_menu' => 'admin_menu',
            'admin_enqueue_scripts' => 'admin_enqueue_scripts',
            'current_screen' => 'current_screen',
            'admin_notices' => 'create_uploads_directory',
            'edit_form_after_title' => 'codemirror_editor',
            'add_meta_boxes' => 'add_meta_boxes',
            'save_post' => 'options_save_meta_box_data',
            'trashed_post' => 'trash_post',
            'untrashed_post' => 'trash_post',
            'admin_post_wpscs-autosave' => 'ajax_autosave',
            'wp_loaded' => 'compatibility_shortcoder',
            'wp_ajax_wpscs_active_code' => 'wp_ajax_wpscs_active_code',
        );
        foreach( $actions as $_key => $_value ) {
            add_action( $_key, array( $this, $_value ) );
        }

        $settings = get_option('wpscs_settings');
        $this->theme = isset($settings['wpscs_editor_theme']) ? $settings['wpscs_editor_theme'] : $this->themeselect;
        if ( $this->theme == 'default' ) {
            $this->theme = '';
        }

        add_action( 'manage_code-studio_posts_custom_column', array( $this, 'manage_posts_columns' ), 10, 2 );
        add_action( 'save_post', array( $this, 'revision_save_meta_box_data' ), 10, 2 );
    }

    function ajax_autosave() {
        check_admin_referer( 'wpscs-nonce' );
        if ( ! isset( $_POST['content'] ) || ! isset( $_POST['title'] ) ) {
            $ajax = new WP_AJAX_Response( array('data' => -1 ) );
            $ajax->send();
            return;
        }
        if ( isset( $_POST['data'] ) ) {
            wp_autosave( $_POST );
            $ajax = new WP_AJAX_Response( array('data' => $_POST['data'] . ' => ' . $_POST['data2'] ) );
            $ajax->send();
            return;
        }
    }
    function admin_menu() {
        $menu_slug = 'edit.php?post_type=code-studio';
        $submenu_slug = 'post-new.php?post_type=code-studio';
        $title = __('Add CSS Code');
        add_submenu_page( $menu_slug, $title, $title, 'manage_options', $submenu_slug .'&language=css');
        $title = __('Add Javascript Code');
        add_submenu_page( $menu_slug, $title, $title, 'manage_options', $submenu_slug . '&language=js');
        $title = __('Add  HTML Code');
        add_submenu_page( $menu_slug, $title, $title, 'manage_options', $submenu_slug . '&language=html');
        /*
        $title = __('Add PHP');
        add_submenu_page( $menu_slug, $title, $title, 'manage_options', $submenu_slug . '&language=php');
        */
        remove_submenu_page( $menu_slug, $submenu_slug);

    }

    public function admin_enqueue_scripts( $hook ) {
        $screen = get_current_screen();
        if ( $screen->post_type != 'code-studio' )
            return false;
        $a = plugins_url( '/', WPSCS_PLUGIN_FILE). 'assets';
        $cm = $a . '/codemirror';
        $v = WPSCS_VERSION;

        // remove the codemirror script if Toolset Types if active
        global $wp_scripts;
        if ( is_plugin_active( 'types/wpcf.php' ) ) {
            if ( isset( $wp_scripts->registered['toolset-codemirror-script'] ) ) {
                unset( $wp_scripts->registered['toolset-codemirror-script'] );
            }
        }

        wp_enqueue_script( 'tipsy', $a . '/jquery.tipsy.js', array('jquery'), $v, false );
        wp_enqueue_style( 'tipsy', $a . '/tipsy.css', array(), $v );
        wp_register_script( 'wpscs_admin', $a . '/wpscs_admin.js', array('jquery', 'tipsy', 'jquery-ui-resizable'), $v, false );
        wp_localize_script( 'wpscs_admin', 'WPSCS', $this->cm_localize() );
        wp_enqueue_script( 'wpscs_admin' );
        wp_enqueue_style( 'wpscs_admin', $a . '/wpscs_admin.css', array(), $v );

        if ( $hook == 'post-new.php' || $hook == 'post.php' ) {
            wp_enqueue_style( 'jquery-ui', 'https://code.jquery.com/ui/1.12.0/themes/base/jquery-ui.css', array(), $v );
            wp_enqueue_script( 'addon_merge_match_patch', $cm . '/diff_match_patch.js', array( 'jquery' ), $v, false );
            wp_enqueue_script( 'codemirror', $cm . '/codemirror-compressed.js', array( 'jquery' ), $v, false);
            wp_enqueue_style( 'codemirror', $cm . '/codemirror-compressed.css', array(), $v );
            wp_enqueue_script( 'wpscs_admin_url_rules', $a . '/wpscs_admin-url_rules.js', array('jquery'), $v, false );

            $cmm = $cm . '/mode/';
            wp_enqueue_script('cm-xml', $cmm . 'xml/xml.js', array('codemirror'), $v, false);
            wp_enqueue_script('cm-js', $cmm . 'javascript/javascript.js', array('codemirror'), $v, false);
            wp_enqueue_script('cm-css', $cmm . 'css/css.js', array('codemirror'), $v, false);
            wp_enqueue_script('cm-htmlmixed', $cmm . 'htmlmixed/htmlmixed.js', array('codemirror'), $v, false);
            wp_enqueue_script('cm-clike', $cmm . 'clike/clike.js', array('codemirror'), $v, false);
            wp_enqueue_script('cm-php', $cmm . 'php/php.js', array('codemirror'), $v, false);

            $cma = $cm . '/addon/';
            wp_enqueue_script('cm-dialog', $cma . 'dialog.js', array('codemirror'), $v, false);
            wp_enqueue_script('cm-search', $cma . 'search.js', array('codemirror'), $v, false);
            wp_enqueue_script('cm-searchcursor', $cma . 'searchcursor.js', array('codemirror'), $v, false);
            wp_enqueue_script('cm-jump-to-line', $cma . 'jump-to-line.js', array('codemirror'), $v, false);
            wp_enqueue_style('cm-matchesonscrollbar', $cma . 'matchesonscrollbar.css', array(), $v );
            wp_enqueue_style('cm-dialog', $cma . 'dialog.css', array(), $v );
            wp_enqueue_style('cm-matchesonscrollbar', $cma . 'matchesonscrollbar.css', array(), $v );

            wp_enqueue_script( 'wpscs_scrollbars', $cm . '/addon/scroll/simplescrollbars.js', array('codemirror'), $v, false );
            wp_enqueue_style( 'wpscs_scrollbars', $cm . '/addon/scroll/simplescrollbars.css', array(), $v );
        }

        if ( !empty($this->theme) ) {
            wp_enqueue_style( 'cmt-' . $this->theme, $cm . '/theme/' . $this->theme . '.css', array(), $v );
        }
    }

    public function cm_localize() {
        $vars = array(
            'scroll' => '1',
            'theme' => $this->theme,
        );
        if ( ! function_exists( 'is_plugin_active' ) ) {
            require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        }

        // Plugins which conflict with the CodeMirror editor
        $conflicting_plugins = array(
            'advanced-ads-code-highlighter/advanced-ads-code-highlighter.php',
            'wp-custom-backend-css/wp-custom-backend-css.php',
            'custom-css-whole-site-and-per-post/h5ab-custom-styling.php',
        );

        foreach( $conflicting_plugins as $_plugin ) {
            if ( is_plugin_active( $_plugin ) ) {
                $vars['scroll'] = '0';
            }
        }

        return $vars;
    }


    public function add_meta_boxes() {
        if ( isset( $_GET['post'] ) ) {
            $language = wpscs_get_options( $_GET['post'] );
            $language = $language['language'];
        } else {
            $language = isset( $_GET['language'] ) ? $_GET['language'] : 'css';
        }

        if ( $language == 'html' ) {
            add_meta_box('custom-code-options', __('Options '), array( $this, 'custom_code_options_meta_box_callback_html'), 'code-studio', 'side', 'low');
        } elseif ( $language =='php' ){
            add_meta_box('custom-code-options', __('Options '), array($this, 'custom_code_options_meta_box_callback_php'), 'code-studio', 'side', 'low' );
        } else {
            add_meta_box('custom-code-options', __('Options'), array( $this, 'custom_code_options_meta_box_callback'), 'code-studio', 'side', 'low');
        }

        remove_meta_box( 'slugdiv', 'code-studio', 'normal' );
    }


    function current_screen( $current_screen ) {
        if ( $current_screen->post_type != 'code-studio' ) {
            return false;
        }
        if ( $current_screen->base == 'post' ) {
            add_action( 'admin_head', array( $this, 'current_screen_post' ) );
        }
        if ( $current_screen->base == 'edit' ) {
            add_action( 'admin_head', array( $this, 'current_screen_edit' ) );
        }
    }

    function add_new_buttons() {
        $current_screen = get_current_screen();
        if ( (isset($current_screen->action ) && $current_screen->action == 'add') || $current_screen->post_type != 'code-studio' ) {
            return false;
        }
?>
    <div class="updated buttons">
        <a href="post-new.php?post_type=code-studio&language=css" class="custom-btn custom-css-btn">Add CSS code</a>
        <a href="post-new.php?post_type=code-studio&language=js" class="custom-btn custom-js-btn">Add JS code</a>
        <a href="post-new.php?post_type=code-studio&language=html" class="custom-btn custom-js-btn">Add HTML code</a>
        <!-- <a href="post-new.php?post_type=code-studio&language=php" class="custom-btn custom-php-btn">Add PHP code</a> -->
    </div>
<?php
    }



    function manage_custom_posts_columns( $columns ) {
        return array(
            'cb'    => '<input type="checkbox" />',
            'active' => '<span class="wpscs-dashicons dashicons dashicons-star-empty" title="Active"></span>',
            'type'  => 'Type',
            'title' => 'Title',
            'author'  => 'Author',
            'date'  => 'Date',
            'modified'  => 'Modified',
            //'options' => 'Options',
        );
    }

    function manage_posts_columns( $column, $post_id ) {

        $options = wpscs_get_options( $post_id );

        if ( $column == 'type' ) {
            echo '<span class="language language-'.$options['language'].'">' . strtoupper($options['language']) . '</span>';
        }

        if ( $column == 'modified' ) {
            if ( ! isset( $wp_version ) ) {
                include( ABSPATH . WPINC . '/version.php' );
            }

            if ( version_compare( $wp_version, '4.6', '>=' ) ) {
                $m_time = get_the_modified_time( 'G', $post_id );
            } else {
                $m_time = get_the_modified_time( 'G', false, $post_id );
            }

            $time_diff = time() - $m_time;

            if ( $time_diff > 0 && $time_diff < DAY_IN_SECONDS ) {
                $h_time = sprintf( __( '%s ago' ), human_time_diff( $m_time ) );
            } else {
                $h_time = mysql2date( __( 'Y/m/d' ), $m_time );
            }

            echo $h_time;
        }

        if ( $column == 'active' ) {
            $url = wp_nonce_url( admin_url( 'admin-ajax.php?action=wpscs_active_code&code_id=' . $post_id ), 'wpscs-active-code' );
            echo '<a href="' . esc_url( $url ) . '" title="'. __( 'Toggle active' ) . '">';
            if ( wpscs_is_active( $post_id ) ) {
                echo '<span class="dashicons dashicons-star-filled" title="The code is active. Click to inactivate it"></span>';
            } else {
                echo '<span class="dashicons dashicons-star-empty wpscs_row" title="The code is inactive. Click to activate it"></span>';
            }
            echo '</a>';

        }
        /*
        if ( $column == 'options' ) {
            echo $this->options_overview( $post_id, $options );
        }
        */
    }

    function options_overview( $post_id, $options ) {

        if ( $options['language'] == 'html' ) {
            $options_array = wpscs_get_options_meta_html();
        } elseif ( $options['language'] == 'php' ) {
            $options_array = wpscs_get_options_meta_php();
        } else {
            $options_array = wpscs_get_options_meta();
        }

        $output = '';


        if ( $options['language'] == 'html' && $options['type'] == 'shortcode' ) {
            $output .= '<span class="dashicons dashicons-paperclip" rel="tipsy" title="Where on the page: As shortcode"></span> ';
            if ( !isset($options['name']) || empty($options['name'])) {
                return $output . ' Shortcode without id';
            } else {
                return $output .' [wpscs id="'.$options['name'].'"]';
            }
            return;
        }


        foreach( $options_array as $_key => $meta ) {
            if ( ! isset( $options[$_key] ) ) {
                continue;
            }
            $_value = $options[$_key];

            if ( $_key == 'priority' ) {
                echo '<span rel="tipsy" title="Priority ' . $_value .'">pr' . $_value . '</span>';
                continue;
            }

            // For radio values with a dashicon
            if ( isset($meta['values'][$_value] ) && isset( $meta['values'][$_value]['dashicon']) ) {
                $icon = 'dashicons ';
                if ( isset( $meta['values'][$_value]['dashicon'] ) ) {
                    $icon .= 'dashicons-' . $meta['values'][$_value]['dashicon'];
                }

                $title = $meta['title'];
                if ( isset( $meta['values'][$_value]['title'] ) ) {
                    $title .= ' : ' . $meta['values'][$_value]['title'];
                }

                $output .= '<span class="' . $icon . '" rel="tipsy" title="'.$title.'"></span> ';
            }
            // For radio values without a dashicon
            elseif ( isset($meta['values'][$_value] ) ) {
                if ( $_value == 'none' ) {
                    continue;
                }
                if ( isset( $meta['values'][$_value]['title'] ) ) {
                    $text = $meta['values'][$_value]['title'];
                    $title = $meta['title'] . ' : ' . $text;
                    $output .= ' <span rel="tipsy" title="'.$title.'">'.$text.'</span> ';
                } else {
                    $text = $meta['values'][$_value];
                    $output .= '<span class="dashicons dashicons-' . $meta['dashicon']. '" rel="tipsy" title="'.$text.'"></span> ';
                }
            }
            // For checkboxes
            elseif ( $_value == true ) {
                $title = $meta['title'];
                $icon = '';
                if ( isset( $meta['dashicon'] ) ) {
                    $icon = 'dashicons dashicons-' . $meta['dashicon'];
                }
                $output .= '<span class="' . $icon . '" rel="tipsy" title="'.$title.'"></span> ';
            }
        }
        return $output;
    }

    function wp_ajax_wpscs_active_code() {
        if ( ! isset( $_GET['code_id'] ) ) die();

        if ( check_admin_referer( 'wpscs-active-code' ) ) {
            $code_id = absint( $_GET['code_id'] );

            if ( 'code-studio' === get_post_type( $code_id ) ) {
                $active = get_post_meta($code_id, '_active', true );
                if ( $active === false || $active === '' ) {
                    $active = 'yes';
                }
                update_post_meta( $code_id, '_active', $active === 'yes' ? 'no' : 'yes' );
                wpscs_build_search_tree();
            }
        }

        wp_safe_redirect( wp_get_referer() ? remove_query_arg( array( 'trashed', 'untrashed', 'deleted', 'ids' ), wp_get_referer() ) : admin_url( 'edit.php?post_type=code-studio' ) );
        die();
    }

    function current_screen_edit() {
        ?>
        <script type="text/javascript">
             /* <![CDATA[ */
            jQuery(window).ready(function($){
                var h1 = 'Code Studio ';
                h1 += '<a href="post-new.php?post_type=code-studio&language=css" class="page-title-action css-add">Add CSS Code</a>';
                h1 += '<a href="post-new.php?post_type=code-studio&language=js" class="page-title-action js-add">Add JS Code</a>';
                h1 += '<a href="post-new.php?post_type=code-studio&language=html" class="page-title-action html-add">Add HTML Code</a>';
                /* h1 += '<a href="post-new.php?post_type=code-studio&language=php" class="page-title-action php-add">Add PHP Code</a>';*/
                $("#wpbody-content h1").html(h1);
            });

        </script>
        <?php
    }

    function current_screen_post() {

        $this->remove_unallowed_metaboxes();

        if ( isset( $_GET['post'] ) ) {
            $action = 'Edit';
            $language = wpscs_get_options( $_GET['post'] );
            $language = $language['language'];
        } else {
            $action = 'Add';
            $language = isset( $_GET['language'] ) ? $_GET['language'] : 'css';
        }

        $title = $action . ' ' . strtoupper( $language ) . ' code';

        if ( $action == 'Edit' ) {
            $title .= ' <a href="post-new.php?post_type=code-studio&language=css" class="page-title-action css-add">Add CSS Code</a> ';
            $title .= '<a href="post-new.php?post_type=code-studio&language=js" class="page-title-action js-add">Add JS Code</a>';
            $title .= '<a href="post-new.php?post_type=code-studio&language=html" class="page-title-action html-add">Add HTML Code</a>';
            /* $title .= '<a href="post-new.php?post_type=code-studio&language=php" class="page-title-action php-add">Add PHP Code</a>'; */
        }

        ?>
        <script type="text/javascript">
             /* <![CDATA[ */
            jQuery(window).ready(function($){
                $("#wpbody-content h1").html('<?php echo $title; ?>');
                $("#message.updated.notice").html('<p>Code updated</p>');
            });
            /* ]]> */
        </script>
        <?php
    }

    function remove_unallowed_metaboxes() {
        global $wp_meta_boxes;
        $allowed = array( 'submitdiv', 'custom-code-options' );
        $allowed = apply_filters( 'code-studio-meta-boxes', $allowed );
        foreach( $wp_meta_boxes['code-studio']['side'] as $_priority => $_boxes ) {
            foreach( $_boxes as $_key => $_value ) {
            if ( ! in_array( $_key, $allowed ) ) {
                unset( $wp_meta_boxes['code-studio']['side'][$_priority][$_key] );
            }
            }
        }
    }

    public function codemirror_editor( $post ) {
        $current_screen = get_current_screen();

        if ( $current_screen->post_type != 'code-studio' ) {
            return false;
        }

        $settings = get_option('wpscs_settings');
        $editor_theme = isset($settings['wpscs_editor_theme']) ? $settings['wpscs_editor_theme'] : $this->themeselect;

        if ( empty( $post->title ) && empty( $post->post_content ) ) {
            $new_post = true;
            $language = isset( $_GET['language'] ) ? $_GET['language'] : 'css';
        } else {
            $new_post = false;
            if ( ! isset( $_GET['post'] ) ) $_GET['post'] = $post->id;
            $language = wpscs_get_options( $_GET['post'] );
            $language = $language['language'];
        }

        switch ( $language ) {
            case 'js' :
                if ( $new_post ) {
                    $post->post_content = '' . PHP_EOL;
                }
                $code_mirror_mode = 'text/javascript';
                $code_mirror_before = '<script type="text/javascript">';
                $code_mirror_after = '</script>';
                break;
           case 'html' :
                if ( $new_post ) {
                    $post->post_content = '' . PHP_EOL;

                }
                $code_mirror_mode = 'application/x-httpd-php';
                $code_mirror_before = '';
                $code_mirror_after = '';
                break;

            case 'php' :
                if ( $new_post ) {
                    $post->post_content = '
---
NOTE:  Currently the PHP is BROKEN... Do not use till I fix it.  If you accidentally use this area
       right now it will REVERT to a CSS file to make sure that we do not blow up the site.
---
                    ' . PHP_EOL;
                }
                $code_mirror_mode = 'php';
                $code_mirror_before = '<?php';
                $code_mirror_after = '?>';

                break;
            default :
                if ( $new_post ) {
                    $post->post_content = '' . PHP_EOL;

                }
                $code_mirror_mode = 'text/css';
                $code_mirror_before = '<style type="text/css">';
                $code_mirror_after = '</style>';

        }
            ?>
              <form style="position: relative; margin-top: .5em;">
                <div id="codemirror_all">
                <div class="code-mirror-buttons">
                </div>
                <div class="code-mirror-before <?php if (!empty($this->theme) ) echo 'cm-before-' . $this->theme; ?>"><div><?php echo htmlentities( $code_mirror_before );?></div></div>
                <textarea class="wp-editor-area" id="content" mode="<?php echo htmlentities($code_mirror_mode); ?>" name="content"><?php echo htmlentities( $post->post_content); ?></textarea>
                <div class="code-mirror-after <?php if (!empty($this->theme) ) echo 'cm-after-' . $this->theme; ?>"><div><?php echo htmlentities( $code_mirror_after );?></div></div>
                <table id="post-status-info"><tbody><tr>
                    <td class="autosave-info">
                    <span class="autosave-message">&nbsp;</span>
                <?php
                    if ( 'auto-draft' != $post->post_status ) {
                        echo '<span id="last-edit">';
                    if ( $last_user = get_userdata( get_post_meta( $post->ID, '_edit_last', true ) ) ) {
                        printf(__('Last edited by %1$s on %2$s at %3$s'), esc_html( $last_user->display_name ), mysql2date(get_option('date_format'), $post->post_modified), mysql2date(get_option('time_format'), $post->post_modified));
                    } else {
                        printf(__('Last edited on %1$s at %2$s'), mysql2date(get_option('date_format'),      $post->post_modified), mysql2date(get_option('time_format'), $post->post_modified));
                    }
                    echo '</span>';
                } ?>
                    </td>
                </tr></tbody></table>
                <input type="hidden" id="update-post_<?php echo $post->ID ?>" value="<?php echo wp_create_nonce('update-post_'. $post->ID ); ?>" />
                <input type="hidden" id="editor_theme" name="editor_theme" value="<?php echo $editor_theme; ?>" />
                </div>
              </form>
    <?php
    }

    function custom_code_options_meta_box_callback( $post ) {
            $options = wpscs_get_options( $post->ID );
            $meta = wpscs_get_options_meta();
            if ( isset( $_GET['language'] ) ) {
                $options['language'] = $_GET['language'];
            }
            wp_nonce_field( 'options_save_meta_box_data', 'code-studio_meta_box_nonce' );
            ?>
            <div class="options_meta_box">
            <?php
            $output = '';
            foreach( $meta as $_key => $a ) {
                // Hide Pre-processors for JavaScript
                if ( $options['language'] == 'js' && $_key == 'preprocessor' ) {
                    continue;
                }
                $output .= '<h4>' . $a['title'] . '</h4>' . PHP_EOL;
                $output .= $this->render_input( $_key, $a, $options);
            }
            echo $output;
            ?>
            <input type="hidden" name="custom_code_language" value="<?php echo $options['language']; ?>" />
            <div style="clear: both;"></div>
            </div>
            <?php
    }

    function options_save_meta_box_data( $post_id ) {
        if ( ! isset( $_POST['code-studio_meta_box_nonce'] ) ) {
            return;
        }
        if ( ! wp_verify_nonce( $_POST['code-studio_meta_box_nonce'], 'options_save_meta_box_data' ) ) {
            return;
        }
        if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
            return;
        }
        if ( isset( $_POST['post_type'] ) && 'code-studio' != $_POST['post_type'] ) {
            return;
        }
        $defaults = wpscs_get_options();
        if ( $_POST['custom_code_language'] == 'html' ) {
            $defaults = wpscs_get_options( $post_id, 'html');
        }
        $options = array();
        foreach( $defaults as $_field => $_default ) {
            $options[ $_field ] = isset( $_POST['custom_code_'.$_field] ) ? $_POST['custom_code_'.$_field] : $_default;
        }
        if ( $options['language'] === 'html' && $options['type'] === 'shortcode' ) {
            $options['name'] = trim($_POST['custom_code_name']);
        }
        update_post_meta( $post_id, 'options', $options );
        // Save the Custom Code in a file in `wp-content/uploads/code-studio`
        if ( $options['language'] != 'html' ) {
            $filename = $post_id . '.' . $options['language'];
            wpscs_save_code_file( $_POST['content'], $options, $filename );
        }
        wpscs_build_search_tree();
    }

    function custom_code_options_meta_box_callback_html( $post ) {
        $options = wpscs_get_options( $post->ID, 'html');
        $meta = wpscs_get_options_meta_html();
        if ( isset( $_GET['language'] ) ) {
            $options['language'] = $_GET['language'];
        }
        wp_nonce_field( 'options_save_meta_box_data', 'code-studio_meta_box_nonce' );
        ?>
        <div class="options_meta_box">
        <?php
        $output = '';
        foreach( $meta as $_key => $a ) {
            $output .= '<h4>' . $a['title'] . '</h4>' . PHP_EOL;
            $output .= $this->render_input( $_key, $a, $options);
        }
        echo $output;
        ?>
        <input type="hidden" name="custom_code_language" value="<?php echo $options['language']; ?>" />
        <div style="clear: both;"></div>
        </div>
        <?php
    }

    function custom_code_options_meta_box_callback_php( $post ) {
        $options = wpscs_get_options( $post->ID, 'php' );
        $meta = wpscs_get_options_meta_php();
        if ( isset( $_GET['language'] )){
            $options['language'] = $_GET['language'];
        }
        ?>
        <div class="options_meta_box">
        <?php
        $output = '';
        foreach( $meta as $_key => $a ) {
            $output .= '<h4>' . $a['title'] . '</h4>' . PHP_EOL;
            $output .= $this->render_input( $_key, $a, $options );
        }
        echo $output;
        ?>

        <div style="clear:both;height:300px;"></div>
        </div>
        <?php
    }

    function create_uploads_directory() {
        $current_screen = get_current_screen();

        if ( $current_screen->base != 'post' || $current_screen->post_type != 'code-studio' ) {
            return false;
        }

        $dir = WPSCS_UPLOAD_DIR;

        // Create the dir if it doesn't exist
        if ( ! file_exists( $dir ) ) {
            wp_mkdir_p( $dir );
        }

        // Show a message if it couldn't create the dir
        if ( ! file_exists( $dir ) ) : ?>
             <div class="notice notice-error is-dismissible">
             <p>The <b>code-studio</b> directory could not be created</p>
             <p>Please run the following commands in order to make the directory: <br /><strong>mkdir <?php echo $dir; ?>; </strong><br /><strong>chmod 777 <?php echo $dir; ?>;</strong></p>
            </div>
        <?php return; endif;

        // Show a message if the dir is not writable
        if ( ! wp_is_writable( $dir ) ) : ?>
             <div class="notice notice-error is-dismissible">
             <p>The <b><?php echo $dir; ?></b> directory is not writable, therefore the CSS and JS files cannot be saved.</p>
             <p>Please run the following command to make the directory writable:<br /><strong>chmod 777 <?php echo $dir; ?> </strong></p>
            </div>
        <?php return; endif;

        // Write a blank index.php
        if ( ! file_exists( $dir. '/index.php' ) ) {
            $content = '<?php' . PHP_EOL . '// Silence is very very quiet.';
            @file_put_contents( $dir. '/index.php', $content );
        }
    }

    function revision_save_meta_box_data( $post_id, $post ) {

        $parent_id = wp_is_post_revision( $post_id );
        if ( ! $parent_id ) {
            return;
        }
        $parent  = get_post( $parent_id );
        if ( $parent->post_type != 'code-studio' ) {
            return;
        }
        $options = get_post_meta( $parent->ID, 'options', true );
        if ( false !== $options )
            add_metadata( 'post', $post_id, 'options', $options );
    }

    function trash_post( $post_id ) {
        wpscs_build_search_tree();
    }

    //Compatibility Issue
    function compatibility_shortcoder() {
        ob_start( array( $this, 'compatibility_shortcoder_html' ) );
    }
    function compatibility_shortcoder_html( $html ) {
        if ( strpos( $html, 'QTags.addButton' ) === false ) return $html;
        if ( strpos( $html, 'codemirror/codemirror-compressed.js' ) === false ) return $html;

        return str_replace( 'QTags.addButton', '// QTags.addButton', $html );
    }

    function render_input( $_key, $a, $options ) {
        $name = 'custom_code_' . $_key;
        $output = '';

        // Show radio type options
        if ( $a['type'] === 'radio' ) {
            $selected = '';
            $output .= '<div class="radio-group">' . PHP_EOL;
            foreach( $a['values'] as $__key => $__value ) {
                $id = $name . '-' . $__key;
                $dashicons = isset($__value['dashicon'] ) ? 'dashicons-before dashicons-' . $__value['dashicon'] : '';
                $selected = ( $__key == $options[$_key] ) ? ' checked="checked" ' : '';
                $output .= '<input type="radio" '. $selected.'value="'.$__key.'" name="'.$name.'" id="'.$id.'">' . PHP_EOL;
                $output .= '<label class="'.$dashicons.'" for="'.$id.'"> '.$__value['title'].'</label><br />' . PHP_EOL;
            }

            if ( $_key === 'type' && $options['type'] === 'shortcode' ) {
                $output .= '<div id="custom_code_name_div"><label for="custom_code_name">Shortcode id: </label> <input type="text" name="custom_code_name" id="custom_code_name" value="'.$options['name'].'" /></div>';
            }

            $output .= '</div>' . PHP_EOL;
        }

        // Show checkbox type options
        if ( $a['type'] == 'checkbox' ) {
            $dashicons = isset($a['dashicon'] ) ? 'dashicons-before dashicons-' . $a['dashicon'] : '';
            $selected = ( isset($options[$_key]) && $options[$_key] == '1') ? ' checked="checked" ' : '';
            $output .= '<div class="radio-group">' . PHP_EOL;
            $output .= '<input type="checkbox" '.$selected.' value="1" name="'.$name.'" id="'.$name.'">' . PHP_EOL;
            $output .= '<label class="'.$dashicons.'" for="'.$name.'"> '.$a['title'].'</label>';
            $output .= '</div>' . PHP_EOL;
        }

        // Show text type options
        if ( $a['type'] == 'text' ) {
            $output .= '<div class="radio-group">' . PHP_EOL;
            $output .= '<input type="text" value="'.$options[$_key].'" name="'.$name.'" id="'.$name.'">' . PHP_EOL;
            $output .= '</div>'. PHP_EOL;

        }


        // Show select type options
        if ( $a['type'] == 'select' ) {
            $output .= '<div class="radio-group">' . PHP_EOL;
            $output .= '<select name="'.$name.'" id="'.$name.'">' . PHP_EOL;
            foreach( $a['values'] as $__key => $__value ) {
                $selected = ( isset($options[$_key]) && $options[$_key] == $__key) ? ' selected="selected"' : '';
                $output .= '<option value="'.$__key.'"'.$selected.'>' . $__value . '</option>' . PHP_EOL;
            }
            $output .= '</select>' . PHP_EOL;
            $output .= '</div>' . PHP_EOL;
        }

        return $output;

    }

}

return new WPSCodeStudio_Admin();