<?php
if ( ! defined( 'ABSPATH' ) ) { exit; }

class WPSCodeStudio_URLRules {

    public function __construct() {
        add_action( 'add_meta_boxes', array( $this, 'add_meta_boxes' ) );
        add_action( 'save_post', array( $this, 'save_meta_box_data' ) );
    }

    function get_filters() {
        return $filters = array(
            'all' => __('Entire Website'),
            'wp-conditional' => __('WP Conditional Tags ..'),
        );
    }

    function add_meta_boxes() {
        add_meta_box( 'url-rules', __('Conditional Logic Options'), array( $this, 'url_rules_meta_box_callback' ), 'code-studio', 'normal', 'high' );
    }

    function url_rules_meta_box_callback( $post ) {

        $filters_html = '';
        $filters = $this->get_filters();
        foreach( $filters as $_key => $_value ) {
            $filters_html .= '<option value="'.$_key.'">' . $_value . '</option>';
        }
        $applied_filters = get_post_meta( $post->ID, 'urls', true );
        if ( ! $applied_filters ) {
            $applied_filters = '[{"value":"","type":"all","index":1}]';
        } else {
            $applied_filters = json_decode($applied_filters );
            foreach( $applied_filters as $_key => $_filter ) {
                $applied_filters[$_key]->value = str_replace('"', '\'', $_filter->value);
                $applied_filters[$_key]->value = htmlspecialchars($_filter->value, ENT_QUOTES);
            }
            $applied_filters = json_encode($applied_filters);
        }

?>
        <input type="hidden" name="scan_anchor_filters" id="wplnst-scan-anchor-filters" value='<?php echo $applied_filters; ?>' />
        <table id="wplnst-elist-anchor-filters" class="wplnst-elist" cellspacing="0" cellpadding="0" border="0" data-editable="true" data-label=""></table>
        <select id="wplnst-af-new-type"><?php echo $filters_html ?></select>&nbsp;
        <input id="wplnst-af-new" type="text" class="regular-text" value="" placeholder="Do not end your statement with an ';'" style="display: none;" />&nbsp;
        <input class="button button-primary" type="button" id="wplnst-af-new-add" value="<?php _e('Add Rule'); ?>" /></td>
        <div class="conditional-logic-help">
            <h4>Conditional Logic Help</h4>
            <div class="conditional-logic-help-content">
            <p>When choosing the "WordPress Conditional Tag" option there are several things to remember:<br>
                <ul>
                    <li>Do not end a conditional statement with a semi-colon. (;)</li>
                    <li>If you want to combine statements where all statements must be true, separate statements with a double AND sign. (&&)</li>
                    <li>If you want to combine statements where at least one of the statements is true, separate statements with a double vertical pipe. (||)</li>
                    <li>If you want to combine statements where at least one of the statements is true but not all statements true, separate statements with an Xor. (XOR)</li>
                    <li>If you want to opposite result of the statement, precede the statement an exclamation point. (!)</li>
                    <li>For more information -> <a href="https://codex.wordpress.org/Conditional_Tags" target="_blank">WordPress Codex</a></li>
                </ul>
            </p>
            </div>
        </div>

<?php
    }

    function save_meta_box_data( $post_id ) {
        if ( ! isset( $_POST['code-studio_meta_box_nonce'] ) ) {
            return;
        }
        if ( ! wp_verify_nonce( $_POST['code-studio_meta_box_nonce'], 'options_save_meta_box_data' ) ) {
            return;
        }
        if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
            return;
        }
        if ( isset( $_POST['post_type'] ) && 'code-studio' != $_POST['post_type'] ) {
            return;
        }
        update_post_meta( $post_id, 'urls', $_POST['scan_anchor_filters'] );
        $this->build_url_tree();
    }

    private function build_url_tree() {
        $posts = query_posts( 'post_type=code-studio&post_status=publish&nopaging=true' );
        if ( ! is_array( $posts ) || count( $posts ) == 0 ) {
            return false;
        }
        $filters = $this->get_filters();
        $tree = array();
        foreach ( $posts as $_post ) {
            $rules = get_post_meta( $_post->ID, 'urls', true );
            if ( ! $rules ) {
                $rules = '[{"value":"","type":"all","index":1}]';
            }
            $rules = $this->check_array_json( $rules );
            $options = wpscs_get_options( $_post->ID );
            if ( ! isset( $options['language'] ) ) continue;
            if ( $options['language'] ==  'html' ) {
                $file = $_post->ID;
            } else {
                $file = $_post->ID . '.' . $options['language'];
            }
            foreach( $rules as $_key => $_rule ) {
                if ( empty( $_rule['value' ] ) ) {
                    $tree[$_rule['type'] ][] = $file;
                } else {
                    $tree[$_rule['type']][$_rule['value']][] = $file;
                }
            }
        }
        update_option( 'code-studio-urls', $tree );
    }

    public function check_array_json($array, $default = array()) {
        $value = @json_decode($array, true);
        return (empty($value) || !is_array($value))? $default : $value;
    }

}

return new WPSCodeStudio_URLRules();
