<?php
if ( ! defined( 'ABSPATH' ) ) { exit; }

class WPSCodeStudio_ShowCodes {

    var $first_page = '';
    var $search_tree;
    var $search_tree_raw;
    var $search_tree_html;
    var $shortcodes;
    var $upload_dir;
    var $upload_url;
    var $preview_id;

    function set_value($var, $value ) {
        $this->$var = $value;
    }

    function wp_conditional_tags( $allowed_codes = array(), $rules = array() ) {
        if ( ! isset($rules['wp-conditional'] ) ) return $allowed_codes;

        $add_rules = array();
        $remove_rules = array();
        $knownconditionals = array('is_home()', 'is_front_page()','is_admin()','is_network_admin()','is_admin_bar_showing()','is_single(','is_sticky(','is_post_type_hierarchical(','is_post_type_archive(','is_comments_popup()','comments_open()','pings_open()','is_page(','is_page_template(','is_category(','in_category(','is_tag(','has_tag(','is_tax(','has_term(','term_exists(','is_taxonomy_hierarchical(','taxonomy_exists(','is_author(','is_multi_author()','is_date()','is_year()','is_month()','is_day()','is_time()','is_new_day()','is_archive()','is_search()','is_404()','is_paged()','is_attachment()','wp_attachment_is_image(','is_local_attachment(','is_singular(','post_type_exists(','is_main_query()','is_feed()','is_trackback()','is_preview()','has_excerpt(','has_nav_menu()','in_the_loop()','is_dynamic_sidebar()','is_active_sidebar()','is_active_widget(','is_blog_installed()','is_rtl()','is_multisite()','is_main_site()','is_super_admin()','is_user_logged_in()','email_exists(','username_exists(','is_plugin_active(','is_child_theme()','is_plugin_inactive(','is_plugin_active_for_network(','current_theme_supports(','has_post_thumbnail(','wp_script_is(','is_customize_preview()');
        foreach( $rules['wp-conditional'] as $_rule => $_codes ) {
            $_rule = stripslashes( trim( $_rule ) );
            if ( empty( $_rule ) ) continue;
            if ( stripos( $_rule, 'return' ) === false ) {
                $_rule = 'return(' . $_rule . ');';
            }
            foreach ($knownconditionals as $conlogic){
	            if ( strpos($_rule, $conlogic) !== false) {

		            if ( eval( $_rule ) ) {
		                $add_rules = array_merge( $add_rules, $_codes );
		            } else {
		                $remove_rules = array_merge( $remove_rules, $_codes );
		            }
		        }
		    }
        }
        $add_rules = array_diff( $add_rules, $remove_rules);

        $allowed_codes = array_merge( $allowed_codes, $add_rules );
        $allowed_codes = array_diff( $allowed_codes, $remove_rules );

        return $allowed_codes;
    }


    function url_rules( $uri, $rules = array() ) {

        if ( $rules === false ) return false;

        if ( ! $rules || count( $rules ) == 0 ) {
            return array();
        }

        $allowed_codes = array();
        $filters = array(
            'all' => 'array_merge',
            'first-page' => 'array_merge',
            'contains' => 'array_merge',
            'equal-to' => 'array_merge',
            'begins-with' => 'array_merge',
            'ends-by' => 'array_merge',
            'not-contains' => 'array_diff',
            'not-equal-to' => 'array_diff',
        );
        foreach( $filters as $_type => $_action ) {
            if ( isset( $rules[$_type] ) ) {
                if ( $_action == 'array_merge' ) {
                    $allowed_codes = array_merge( $allowed_codes, $this->check_url_rules( $_type, $rules[$_type], $uri) );
                } else {
                    $allowed_codes = array_diff( $allowed_codes, $this->check_url_rules( $_type, $rules[$_type], $uri) );
                }
            }
        }

        return array_unique( $allowed_codes );
    }


    function check_url_rules ( $type, $rules, $uri ) {

        if ( $type == 'all' ) return $rules;

        if ( $type == 'first-page' ) {
            $first_page = parse_url( $this->first_page );
            if ( $first_page['path'] == $uri || $first_page['path'] . '/' == $uri ) {
                return $rules;
            }
            return array();
        }


        if ( ! is_array( $rules ) || count( $rules ) == 0 ) return array();

        $all_codes = array();

        foreach( $rules as $_key => $_codes ) {
            $accept = false;
            switch( $type ) {
                case 'contains' :
                case 'not-contains' :
                    if ( strpos( $uri, $_key ) !== false ) $accept = true;
                    break;
                case 'equal-to' :
                case 'not-equal-to' :
                    if ( $uri == $_key ) $accept = true;
                    break;
                case 'begins-with' :
                    if ( strpos( $uri, $_key ) === 0 ) $accept = true;
                    break;
                case 'ends-by' :
                    if ( strpos( strrev( $uri ), strrev( $_key ) ) === 0 ) $accept = true;
                    break;
            }
            if ( $accept ) {
                $all_codes = array_merge( $all_codes, $_codes );
            }
        }
        return $all_codes;
    }

    function filter_search_tree( $search_tree = array(), $allowed_codes  = array()) {

        if ( ! is_array( $search_tree ) || count( $search_tree ) == 0 ) {
            return false;
        }

        if ( ! is_int( key( $search_tree ) ) ) {
            $search_tree[5] = $search_tree;
        }
        ksort( $search_tree );

        if ( $allowed_codes === false ) {
            return $search_tree;
        }

        if ( ! is_array( $allowed_codes ) || count($allowed_codes ) == 0 ) {
            return array();
        }

        foreach( $search_tree as $_priority => $_sub ) {
            foreach( $_sub as $_action => $_codes ) {
                if ( strpos( $_action, 'external' ) !== false ) {
                    foreach( $_codes as $__key => $__code ) {
                        if ( ! in_array( $this->short_filename( $__code ), $allowed_codes ) ) {
                            unset( $_codes[$__key] );
                        }
                    }
                } else {
                    if ( !is_array( $_codes ) ) {
                        $_codes = array();
                    }
                    if ( !is_array( $allowed_codes ) ) {
                        $allowed_codes = array();
                    }
                    $_codes = array_intersect( $_codes, $allowed_codes );
                }
                if ( is_array( $_codes ) && count( $_codes ) > 0 ) {
                    $search_tree[$_priority][$_action] = $_codes;
                } else {
                    unset( $search_tree[$_priority][$_action] );
                }
            }
        }

        return $search_tree;
    }


    function search_tree_for_preview( $search_tree = array(), $preview_id ) {

        if ( ! is_array( $search_tree ) || count( $search_tree ) == 0 ) {
            return false;
        }

        if ( empty($preview_id) ) {
            return $search_tree;
        }

        $transient = get_transient(WPSCS_PREVIEW_PREFIX . $preview_id );

        if ( false == $transient ) {
            add_action( 'wp_head', array( $this, 'alert_preview_expired' ) );
        }

        $post_meta = get_post_meta($transient['post_ID'], 'options', true );

        if ( $post_meta ) {
            $tree_branch = $post_meta['side'] . '-' .$post_meta['language'] . '-' . $post_meta['type'] . '-' . $post_meta['linking'];

            foreach( $search_tree as $priority => $_sub ) {
                if ( ! isset( $_sub[$tree_branch] ) ) continue;
                foreach( $_sub[$tree_branch] as $_key => $_codes ) {
                    if ( strpos( $_codes, $transient['post_ID'] . '.' ) === 0 || $_codes == $transient['post_ID']) {
                        unset( $search_tree[$priority][$tree_branch][$_key] );
                    }
                }
                if ( count( $_sub[$tree_branch] ) == 0 ) {
                    unset( $search_tree[$priority][$tree_branch] );
                }
            }
        }

        $new_tree_branch = $transient['side'] . '-' . $transient['language'] . '-' . $transient['type'] . '-' . $transient['linking'];

        $priority = $transient['priority'];

        $filename = $transient['post_ID'] . '-preview.' . $transient['language'];

        if ( $transient['linking'] == 'external' ) {
            $filename .= '?v=' . rand(1, 10000);
        }

        $search_tree[$priority][$new_tree_branch][] = $filename;

        return $search_tree;
    }

    function alert_preview_expired() {
        echo '<script type="text/javascript"> alert("The preview id you are using is already expired. Please generate the preview again."); </script>' . PHP_EOL;
    }

    function print_code_actions($search_tree = array()) {

        if( ! is_array($search_tree) || count( $search_tree ) == 0 ) {
            return;
        }

        foreach( array( 'wp_head', 'wp_footer', 'admin_head', 'admin_footer' ) as $action ) {
            add_action( $action, array( $this, 'print_' . $action), 30 );
        }
    }

    public function __call( $function, $args ) {
        if ( strstr( $function, 'print_' ) == false ) {
            return false;
        }

        $action = str_replace( 'print_', '', $function );

        foreach( $this->search_tree as $_priority => $_sub ) {
            foreach( $_sub as $_where => $_codes ) {

                if ( $action == 'wp_head' ) {
                    if ( strpos( $_where, 'admin' ) === false && strpos( $_where, 'header' ) !== false ) {
                        $this->show_this_subtree( $_where, $_codes );
                    }
                }

                if ( $action == 'wp_footer' ) {
                    if ( strpos( $_where, 'admin' ) === false && strpos( $_where, 'footer' ) !== false ) {
                        $this->show_this_subtree( $_where, $_codes );
                    }
                }

                if ( $action == 'admin_head' ) {
                    if ( strpos( $_where, 'admin' ) !== false && strpos( $_where, 'header' ) !== false ) {
                        $this->show_this_subtree( $_where, $_codes );
                    }
                }

                if ( $action == 'admin_footer' ) {
                    if ( strpos( $_where, 'admin' ) !== false && strpos( $_where, 'footer' ) !== false ) {
                        $this->show_this_subtree( $_where, $_codes );
                    }
                }
            }
        }
    }

    function show_this_subtree( $_where, $_codes ) {

        $output = '';

        if ( strstr( $_where, 'internal' ) ) {

            $before = '<!-- start Code Studio -->' . PHP_EOL;
            $after = '<!-- end Code Studio -->' . PHP_EOL;
            if ( strpos( $_where, 'css' ) !== false ) {
                $before .= '<style type="text/css">' . PHP_EOL;
                $after = '</style>' . PHP_EOL . $after;
            }
            if ( strpos( $_where, 'js' ) !== false ) {
                $before .= '<script type="text/javascript">' . PHP_EOL;
                $after = '</script>' . PHP_EOL . $after;
            }

            foreach( $_codes as $_post_id ) {
                if ( strstr( $_post_id, 'css' ) || strstr( $_post_id, 'js' ) ) {
                    @include_once( $this->upload_dir . '/' . $_post_id );
                } else {
                    $post = get_post( $_post_id );
                    $output .= $before . $post->post_content . $after;
                }
            }
        }

        if ( strpos( $_where, 'external' ) !== false ) {
            if ( strpos( $_where, 'js' ) !== false ) {
                foreach( $_codes as $_filename ) {
                    $output .= PHP_EOL . "<script type='text/javascript' src='".$this->upload_url . '/' . $_filename."'></script>" . PHP_EOL;
                }
            }

            if ( strpos( $_where, 'css' ) !== false ) {
                foreach( $_codes as $_filename ) {
                    $id = $this->short_filename($_filename) . '-css';
                    $href = $this->upload_url . '/' . $_filename;
                    $output .= PHP_EOL . "<link rel='stylesheet' id='".$id."'  href='".$href."' type='text/css' media='all' />" . PHP_EOL;
                }
            }
        }

        if ( strpos( $_where, 'html' ) !== false ) {
            $is_mobile = $this->is_mobile();
            if ( ( $is_mobile && strpos( $_where, 'desktop' ) === false ) || ( !$is_mobile && strpos( $_where, 'mobile' ) === false ) ) {
                foreach( $_codes as $_post_id ) {
                    if ( strpos( $_post_id, '-preview' ) ) {
                        @include_once( $this->upload_dir . '/' . $_post_id );
                    } else {
                        $post = get_post( $_post_id );
                        $output .= $post->post_content;
                    }
                }
            }
        }

        echo $output;

    }

    function short_filename($filename) {
        return preg_replace( '@\.(css|js)\?v=.*$@', '.$1', $filename );
    }

    function print_html_code_actions($search_tree = array()) {

        if( ! is_array($search_tree) || count( $search_tree ) == 0 ) {
            return;
        }

        foreach( $search_tree as $_key => $_value ) {
            $action = '';
            $_key = str_replace( array('desktop-', 'mobile-', 'both-'), '', $_key );

            $allowed_hooks = array(
                'wp_head', 'wp_footer'
            );

            if ( ! in_array( $_key, $allowed_hooks ) ) {
                continue;
            }

            add_action( $action, array( $this, 'printh_' . $_key ) );
        }
    }

    function add_shortcodes() {
        if ( !is_array( $this->search_tree_raw) || count($this->search_tree_raw) == 0 ) return;

        if ( !isset( $this->search_tree_raw[10] ) ) return;

        if ( !isset( $this->search_tree_raw[10]['shortcode'] ) ) return;

        $shortcodes = array();
        foreach( $this->search_tree_raw[10]['shortcode'] as $shortcode_id ) {
            $shortcode = explode('-', $shortcode_id, 2);

            if ( !is_array($shortcode) || count($shortcode) !== 2 )
                continue;

            $shortcodes[$shortcode[1]] = $shortcode[0];
        }
        $this->shortcodes = $shortcodes;

        add_shortcode( 'wpscs', array( $this, 'print_shortcode' ) );
    }

    function print_shortcode( $atts ) {
        if ( !isset($atts['id']) || !isset($this->shortcodes[$atts['id']] ) ) return;

        $post = get_post( $this->shortcodes[$atts['id']]);
        $wpscs_content_print = $post->post_content;

        if ( count( $atts ) > 1 ) {
            foreach( $atts as $_key => $_value ) {
                $wpscs_content_print = str_replace('{$'.$_key.'}', $_value, $wpscs_content_print );
            }
        }

        ob_start();
        extract($atts);
        eval('?>' . str_ireplace(array('&lt;?php', '?&gt;'), array('<?php', '?>'), $wpscs_content_print ) );
        $wpscs_content_print = ob_get_clean();

        $wpscs_content_print = do_shortcode( $wpscs_content_print );

        return $wpscs_content_print;
    }

    function is_mobile() {
        static $is_mobile = null;

        if ( isset( $is_mobile ) ) {
            return $is_mobile;
        }

        if ( empty($_SERVER['HTTP_USER_AGENT']) ) {
            $is_mobile = false;
        } elseif ( strpos($_SERVER['HTTP_USER_AGENT'], 'Mobile') !== false // many mobile devices (all iPhone, iPad, etc.)
            || strpos($_SERVER['HTTP_USER_AGENT'], 'Android') !== false
            || strpos($_SERVER['HTTP_USER_AGENT'], 'Silk/') !== false
            || strpos($_SERVER['HTTP_USER_AGENT'], 'Kindle') !== false
            || strpos($_SERVER['HTTP_USER_AGENT'], 'BlackBerry') !== false
            || strpos($_SERVER['HTTP_USER_AGENT'], 'Opera Mini') !== false
            || strpos($_SERVER['HTTP_USER_AGENT'], 'Opera Mobi') !== false ) {
                $is_mobile = true;
        } else {
            $is_mobile = false;
        }

        return $is_mobile;
    }


}
