<?php
if ( ! defined( 'ABSPATH' ) ) { exit; }

if ( ! function_exists( 'wpscs_get_options' ) ) {
function wpscs_get_options( $post_id = 0, $language = 'css' ) {
    $default_options = array(
        'type' => 'header',
        'linking' => 'internal',
        'side' => 'frontend',
        'language' => 'css',
        'preprocessor' => 'none',
        'minify' => false,
        'priority' => 5,
    );
    if ( $language == 'html' ) {
        $default_options = array(
            'type' => 'header',
            'linking' => 'both',
            'side' => 'frontend',
            'language' => 'html',
            'priority' => 5,
        );
    }

    if ( empty( $post_id ) || $post_id == 0 ) {
        return $default_options;
    }

    $options = get_post_meta( $post_id );
    if ( empty( $options ) || ! isset ( $options['options'][0] ) ) {
        return $default_options;
    }

    $options = unserialize( $options['options'][0] );

    foreach( $default_options as $_key => $_value ) {
        if ( ! isset($options[$_key] ) ) {
            $options[$_key] = $_value;
        }
    }

    return $options;
}
}

if ( ! function_exists( 'wpscs_save_code_file' ) ) {
function wpscs_save_code_file( $content, $options, $filename = '' ) {

    $content = stripslashes( $content );

    if ( $options['preprocessor'] != 'none' ) {
        $content = wpscs_modify_code('preprocess', $options['preprocessor'], $content );
    }
    if ( $options['minify'] == '1' ) {
        $content = wpscs_modify_code('minify', $options['language'], $content );
    }


    $before = ''; $after = '';
    if ( $options['linking'] == 'internal' ) {
        if ( $options['language'] == 'css' ) {
            $before .= '<style type="text/css">' . PHP_EOL;
            $after = '</style>' . PHP_EOL . $after;
        }
        if ( $options['language'] == 'js' ) {
            if ( ! preg_match( '/<script\b[^>]*>([\s\S]*?)<\/script>/im', $content ) ) {
            $before .= '<script type="text/javascript">' . PHP_EOL;
            $after = '</script>' . PHP_EOL . $after;
            }
        }
    }

    if ( $options['linking'] == 'external' ) {
        $before = '/******* Do not edit this file *******' . PHP_EOL .
        'This code and file are managed by the Code Studio plugin' . PHP_EOL .
        'Last Updated: '.date('M d Y | H:i:s').' */' . PHP_EOL;
    }

    if ( wp_is_writable( WPSCS_UPLOAD_DIR ) ) {
        $file_content = $before . $content . $after;
        @file_put_contents( WPSCS_UPLOAD_DIR. '/' . $filename , $file_content );
    }
}
}

if ( ! function_exists( 'wpscs_modify_code' ) ) {
function wpscs_modify_code($type = 'minify', $option = 'ccs', $content ) {

    if ( $type == 'minify' ) {
        // Minify the CSS
        if ( $option == 'css' ) {
            require_once( 'vendor/CSSmin.php' );
            $obj = new CSSmin();
            try {
                $content = $obj->run( $content );
            } catch( Exception $e ) {
                /* Put the exceptions in WP logs */
            }
        }

        // Minify the JS
        if ( $option == 'js' ) {
            require_once( 'vendor/Minifier.php' );
            try {
                $content = Minifier::minify( $content );
            } catch( Exception $e ) {
                /* Put the exceptions in WP logs */
            }
        }

    }

    if ( $type == 'preprocess' ) {
        if ( $option == 'less' ) {
            require_once( 'vendor/less.inc.php' );
            $compiler = new lessc();
        }

        if ( $option == 'sass' ) {
            require_once( 'vendor/scss.inc.php' );
            $compiler = new scssc();
        }

        if ( isset( $compiler ) ) {
            try {
                return $compiler->compile( $content );
            } catch( Exception $e ) {
                return $content;
            }
        }
    }

    return $content;

}
}

if ( ! function_exists( 'wpscs_build_search_tree' ) ) {
function wpscs_build_search_tree() {

    $posts = query_posts( 'post_type=code-studio&post_status=publish&nopaging=true' );

    $tree = array();
    foreach ( $posts as $_post ) {
        if ( ! wpscs_is_active( $_post->ID ) ) {
            continue;
        }

        $options = wpscs_get_options( $_post->ID );

        $tree_branch = $options['side'] . '-' .$options['language'] . '-' . $options['type'] . '-' . $options['linking'];

        $filename = $_post->ID . '.' . $options['language'];

        if ( $options['linking'] == 'external' ) {
            $filename .= '?v=' . rand(1, 10000);
        }

        if ( $options['language'] == 'html' ) {
            if ( $options['type'] == 'shortcode' ) {
                $tree_branch = 'shortcode';
                $options['priority'] = 10;
                $filename = $_post->ID . '-' . $options['name'];
            } else {
                $tree_branch = $options['side'] . '-html-' . $options['type'] . '-' .$options['linking'];
                $filename = $_post->ID;
            }
        }

        $priority = isset( $options['priority'] ) ? $options['priority'] : 5;

        $tree[$priority][ $tree_branch ][] = $filename;

    }

    update_option( 'code-studio-tree', $tree );
}
}



if ( ! function_exists( 'wpscs_is_active' ) ) {
function wpscs_is_active( $post_id ) {
    return get_post_meta( $post_id, '_active', true ) !== 'no';
}
}


if ( ! function_exists( 'wpscs_get_options_meta' ) ) {
function wpscs_get_options_meta() {
    $options = array(
        'linking' => array(
            'title' => 'How to Include Code',
            'type' => 'radio',
            'default' => 'internal',
            'values' => array(
                'external' => array(
                    'title' => 'Stand Alone File',
                    //'dashicon' => 'media-code',
                ),
                'internal' => array(
                    'title' => 'Output to Page',
                    //'dashicon' => 'editor-alignleft',
                ),
            ),
        ),
        'type' => array(
            'title' => 'Location',
            'type' => 'radio',
            'default' => 'header',
            'values' => array(
                'header' => array(
                    'title' => 'In the Header',
                    //'dashicon' => 'arrow-up-alt2',
                ),
                'footer' => array(
                    'title' => 'In the Footer',
                    //'dashicon' => 'arrow-down-alt2',
                ),
            ),
        ),
        'side' => array(
            'title' => 'Front or Back',
            'type' => 'radio',
            'default' => 'frontend',
            'values' => array(
                'frontend' => array(
                    'title' => 'Public Side',
                    //'dashicon' => 'tagcloud',
                ),
                'admin' => array(
                    'title' => 'Admin Side',
                    //'dashicon' => 'id',
                ),
            ),
        ),
        'preprocessor' => array(
            'title' => 'Use a Preprocessor',
            'type' => 'radio',
            'default' => 'none',
            'values' => array(
                'none' => array(
                    'title' => 'Do not use',
                ),
                'less' => array(
                    'title' => 'Use LESS',
                ),
                'sass' => array(
                    'title' => 'Sass (SCSS syntax)',
                ),
            ),
        ),
        'minify' => array(
            'title' => 'Minify the code',
            'type' => 'checkbox',
            'default' => false,
            //'dashicon' => 'editor-contract',
        ),
        'priority' => array(
            'title' => 'Priority',
            'type' => 'select',
            'default' => 5,
            'dashicon' => 'sort',
            'values' => array(
                1 => '1 (highest)',
                2 => '2',
                3 => '3',
                4 => '4',
                5 => '5',
                6 => '6',
                7 => '7',
                8 => '8',
                9 => '9',
                10 => '10 (lowest)',
            ),
        ),
    );

    return $options;
}
}

if ( ! function_exists( 'wpscs_get_options_meta_php' )) {
function wpscs_get_options_meta_php() {
    $options = array(
        'type' => array(
            'title' => 'This will execute as if it were in the functions.php file of your theme.',
            'type' => 'radio',
            'values' => array(
            ),
        ),
    );
    return $options;
}
}

if ( ! function_exists( 'wpscs_get_options_meta_html' ) ) {
function wpscs_get_options_meta_html() {
    $options = array(
        'type' => array(
            'title' => 'Location',
            'type' => 'radio',
            'default' => 'header',
            'values' => array(
                'header' => array(
                    'title' => 'Header',
                    //'dashicon' => 'arrow-up-alt2',
                ),
                'footer' => array(
                    'title' => 'Footer',
                    //'dashicon' => 'arrow-down-alt2',
                ),
                'shortcode' => array(
                    'title' => 'Create Shortcode',
                    //'dashicon' => 'paperclip',
                ),
            ),
        ),
        'side' => array(
            'title' => 'Front or Back',
            'type' => 'radio',
            'default' => 'frontend',
            'values' => array(
                'frontend' => array(
                    'title' => 'Public Side',
                    //'dashicon' => 'tagcloud',
                ),
                'admin' => array(
                    'title' => 'Admin Side',
                    //'dashicon' => 'id',
                ),
            ),
        ),
        'linking' => array(
            'title' => 'Mobile Control',
            'type' => 'radio',
            'default' => 'both',
            'dashicon' => '',
            'values' => array(
                'desktop' => array(
                    'title' => 'Desktop',
                    //'dashicon' => 'desktop',
                ),
                'mobile' => array(
                    'title' => 'Mobile',
                    //'dashicon' => 'smartphone',
                ),
                'both' => array(
                    'title' => 'Both',
                    //'dashicon' => 'tablet',
                ),
            ),
        ),
        'priority' => array(
            'title' => 'Priority',
            'type' => 'select',
            'default' => 5,
            'dashicon' => 'sort',
            'values' => array(
                1 => '1 (highest)',
                2 => '2',
                3 => '3',
                4 => '4',
                5 => '5',
                6 => '6',
                7 => '7',
                8 => '8',
                9 => '9',
                10 => '10 (lowest)',
            ),
        ),
    );

    return $options;
}
}